import objectdraw.*;
import java.awt.Color;
import java.util.Random;

/*
 * A class that implements a nice looking basketball, this one implementing
 * the Drawable2DInterface from Objectdraw.
 *
 * Jim Teresco, The College of Saint Rose, Spring 2014
 *
 * $Id: NiceBBall.java 2329 2014-02-27 02:46:26Z terescoj $
 */

public class NiceBBall implements Drawable2DInterface {

    // the color to draw the ball
    private static final Color BALL_ORANGE = new Color(250, 85, 10);

    // size and starting angles for cut arc in degrees
    private static final int CUTSIZE = 100;
    private static final int RIGHTCUTSTART = 90 + (180 - CUTSIZE) / 2;
    private static final int LEFTCUTSTART = 270 + (180 - CUTSIZE) / 2;

    // the orange part of the ball
    private FilledOval body;

    // the border of the ball
    private FramedOval border;

    // the two curves on the sides of the ball
    private FramedArc leftCut, rightCut;

    // the vertical and horizontal lines through the ball
    private Line vert, horiz;

    // to pick new random color for ball
    private Random colorPicker = new Random();

    /*
     * Create a new basketball on the given DrawingCanvas.  Note how this class
     * does not know about the default "canvas", since that's available only in
     * classes that extend the WindowController class.
     */
    public NiceBBall(double left, double top, double size, DrawingCanvas aCanvas) {

        // draw the circles that make it look like a ball
        body = new FilledOval(left, top, size, size, aCanvas);
        body.setColor(BALL_ORANGE);
        border = new FramedOval(left, top, size, size, aCanvas);

        // draw the lines and arcs that make it look like a basketball
        rightCut = new FramedArc(left + size * 2 / 3, top, size, size, RIGHTCUTSTART, CUTSIZE, aCanvas);
        leftCut = new FramedArc(left - size * 2 / 3, top, size, size, LEFTCUTSTART, CUTSIZE, aCanvas);
        vert = new Line(left + size / 2, top, left + size / 2, top + size, aCanvas);
        horiz = new Line(left, top + size / 2, left + size, top + size / 2, aCanvas);

    }

    /*
     * Move the ball by specified offsets.
     */
    public void move(double dx, double dy) {
        // move each part of the ball by the offsets provided
        body.move(dx, dy);
        border.move(dx, dy);
        vert.move(dx, dy);
        horiz.move(dx, dy);
        leftCut.move(dx, dy);
        rightCut.move(dx, dy);
    }

    /*
     * Check to see if the ball contains a specified location.
     */
    public boolean contains(Location point) {
        return body.contains(point);
    }

    /*
     * Move the ball to a specified position. 
     */
    public void moveTo(double x, double y) {
        // determine how far away (x,y) is
        double dx, dy;
        dx = x - body.getX();
        dy = y - body.getY();

        // move each part of the ball by the offset,
        //   using our own move method.
        this.move(dx, dy);
    }

    /*
     * Return the x coordinate of ball's corner.
     */
    public double getX() {
        return body.getX();
    }

    /*
     * Return the y coordinate of ball's corner.
     */
    public double getY() {
        return body.getY();
    }
    // we are required by Drawable2DInterface to provide an overlaps method
    // which just checks if the "body" here overlaps the other
    public boolean overlaps(Drawable2DInterface other) {

        return body.overlaps(body);
    }

    // getLocation required by Drawable2DInterface
    public Location getLocation() {

        return body.getLocation();
    }

    // we also must provide getWidth and getHeight
    public double getWidth() {

        return body.getWidth();
    }

    public double getHeight() {

        return body.getHeight();
    }

    // also stacking order methods
    public void sendToFront() {

        body.sendToFront();
        border.sendToFront();
        rightCut.sendToFront();
        leftCut.sendToFront();
        vert.sendToFront();
        horiz.sendToFront();
    }

    public void sendToBack() {

        border.sendToBack();
        rightCut.sendToBack();
        leftCut.sendToBack();
        vert.sendToBack();
        horiz.sendToBack();
        body.sendToBack();
    }

    public void sendBackward() {

        border.sendBackward();
        rightCut.sendBackward();
        leftCut.sendBackward();
        vert.sendBackward();
        horiz.sendBackward();
        body.sendBackward();
    }

    public void sendForward() {

        border.sendForward();
        rightCut.sendForward();
        leftCut.sendForward();
        vert.sendForward();
        horiz.sendForward();
        body.sendForward();
    }

    // standard get and set color needed by Drawable2DInterface
    public void setColor(Color c) {

        body.setColor(c);
    }

    public Color getColor() {

        return body.getColor();
    }

    public void moveTo(Location newLoc) {

        moveTo(newLoc.getX(), newLoc.getY());
    }

    // hiding and showing
    public boolean isHidden() {

        return body.isHidden();
    }

    public void show() {

        border.show();
        rightCut.show();
        leftCut.show();
        vert.show();
        horiz.show();
        body.show();
    }

    public void hide() {

        border.hide();
        rightCut.hide();
        leftCut.hide();
        vert.hide();
        horiz.hide();
        body.hide();
    }

    public DrawingCanvas getCanvas() {

        return body.getCanvas();
    }

    public void addToCanvas(DrawingCanvas c) {

        border.addToCanvas(c);
        rightCut.addToCanvas(c);
        leftCut.addToCanvas(c);
        vert.addToCanvas(c);
        horiz.addToCanvas(c);
        body.addToCanvas(c);
    }

    public void removeFromCanvas() {

        border.removeFromCanvas();
        rightCut.removeFromCanvas();
        leftCut.removeFromCanvas();
        vert.removeFromCanvas();
        horiz.removeFromCanvas();
        body.removeFromCanvas();
    }        
        
}
