import objectdraw.*;
import java.awt.*;
import java.util.Random;

/*
 * A T-shirt object, this one implements the Objectdraw built in
 * interface Drawable2DInterface.
 *
 * Jim Teresco, The College of Saint Rose, Spring 2014
 * Based on example from Williams College CSCI 134
 *
 * $Id: TShirt.java 2329 2014-02-27 02:46:26Z terescoj $
 */

public class TShirt implements Drawable2DInterface {

    // constants to define the size of the shirt
    // note how in this case, all are defined relative
    // to a single "SIZE" that we can change and
    // have all components change in an appropriate
    // proportion.
    private static final double SIZE = 120;

    private static final double SLEEVE_WIDTH = SIZE;
    private static final double SLEEVE_HEIGHT = 0.2*SIZE;

    private static final double BODY_WIDTH = 0.6*SIZE;
    private static final double BODY_HEIGHT = (0.65)*SIZE;

    private static final double BODY_INSET = 0.2*SIZE;

    private static final double NECK_WIDTH = 0.3*SIZE;
    private static final double NECK_HEIGHT = 0.06*SIZE;

    private static final double NECK_INSET = 0.35*SIZE;

    // rectangles that form a border around the shirt
    private FramedRect sleeveTrim, bodyTrim;   
    private FramedOval neckTrim;				

    // rectangles that form the interior color of the shirt
    private FilledRect body, sleeves;	
    private FilledOval neck;					

    // Current color of the shirt
    private Color hue;							

    // Random number generator used to select next object
    private Random generator = new Random();

    // create a new T-shirt with its upper left corner at (x,y).
    public TShirt(double x, double y, DrawingCanvas canvas) {
        
        // create boundary rectangles
        sleeveTrim = new FramedRect(x, y + NECK_HEIGHT/2, SLEEVE_WIDTH, 
            SLEEVE_HEIGHT, canvas);
        bodyTrim = new FramedRect(x + BODY_INSET, y + NECK_HEIGHT/2,
            BODY_WIDTH, BODY_HEIGHT, canvas);

        // create interior rectangles
        sleeves = new FilledRect(x+1, y+NECK_HEIGHT/2+1, SLEEVE_WIDTH-1,
            SLEEVE_HEIGHT-1, canvas);
        body = new FilledRect(x+BODY_INSET+1, y+NECK_HEIGHT/2+1,
            BODY_WIDTH-1, BODY_HEIGHT-1, canvas);

        // give it a neck hole
        neck = new FilledOval(x + NECK_INSET, y, NECK_WIDTH,
            NECK_HEIGHT, canvas);
        neckTrim = new FramedOval(x + NECK_INSET, y, NECK_WIDTH, 
            NECK_HEIGHT, canvas);

        // set the interior color
        setColor();
    }

    // move the t-shirt by specified offsets.
    public void move(double xOffset, double yOffset) {
        
        body.move(xOffset,yOffset);
        neck.move(xOffset,yOffset);
        sleeves.move(xOffset,yOffset);
        bodyTrim.move(xOffset,yOffset);
        sleeveTrim.move(xOffset,yOffset);
        neckTrim.move(xOffset,yOffset);
    }

    // remove the t-shirt from the canvas.
    public void removeFromCanvas() {
        
        body.removeFromCanvas();
        neck.removeFromCanvas();
        sleeves.removeFromCanvas();
        bodyTrim.removeFromCanvas();
        sleeveTrim.removeFromCanvas();
        neckTrim.removeFromCanvas();
    }

    // move the t-shirt to a new upper-left coordinate position
    public void moveTo(double x, double y) {
        
        move(x - sleeves.getX(), y - neck.getY());
    }

    // returns true if the t-shirt contains the point; false otherwise
    public boolean contains(Location pt) {
        return body.contains(pt) || sleeves.contains(pt) || neck.contains(pt);

    }

    // pick a new random color for the shirt
    private void setColor() {

        hue = new Color(generator.nextInt(256),
            generator.nextInt(256),
            generator.nextInt(256));

        body.setColor(hue);
        sleeves.setColor(hue);
        neck.setColor(hue);
    }

    // we are required by Drawable2DInterface to provide an overlaps method
    // which just checks if the essential parts overlap the other object
    public boolean overlaps(Drawable2DInterface other) {
        
        return other.overlaps(body) || other.overlaps(sleeves) || other.overlaps(neck);
    }
    
    // getLocation, getX, getY are required by Drawable2DInterface
    public Location getLocation() {
        
        return new Location(sleeves.getX(), neck.getY());
    }
    
    public double getX() {
        
        return sleeves.getX();
    }
    
    public double getY() {
        
        return neck.getY();
    }
    
    // we also must provide getWidth and getHeight
    public double getWidth() {
        
        return SLEEVE_WIDTH;
    }
    
    public double getHeight() {
        
        return body.getY() + body.getHeight() - neck.getY();
    }
    
    // also stacking order methods
    public void sendToFront() {
        
        sleeveTrim.sendToFront();
        bodyTrim.sendToFront();
        sleeves.sendToFront();
        body.sendToFront();
        neck.sendToFront();
        neckTrim.sendToFront();
       
    }
    
    public void sendToBack() {
        
        neckTrim.sendToBack();
        neck.sendToBack();
        body.sendToBack();
        sleeves.sendToBack();
        bodyTrim.sendToBack();
        sleeveTrim.sendToBack();
    }
    
    public void sendBackward() {
        
        sleeveTrim.sendBackward();
        bodyTrim.sendBackward();
        sleeves.sendBackward();
        body.sendBackward();
        neck.sendBackward();
        neckTrim.sendBackward();
        
    }
    
     public void sendForward() {
        
        sleeveTrim.sendForward();
        bodyTrim.sendForward();
        sleeves.sendForward();
        body.sendForward();
        neck.sendForward();
        neckTrim.sendForward();
        
    }
    
    // standard get and set color needed by Drawable2DInterface
    public void setColor(Color c) {
        
        sleeves.setColor(c);
        body.setColor(c);
        neck.setColor(c);
    }
    
    public Color getColor() {
        
        return sleeves.getColor();
    }
    
    public void moveTo(Location newLoc) {
        
        moveTo(newLoc.getX(), newLoc.getY());
    }
    
    // hiding and showing
    public boolean isHidden() {
        
        return body.isHidden();
    }
    
    public void show() {
        
        body.show();
        sleeves.show();
        neck.show();
        bodyTrim.show();
        neckTrim.show();
        sleeveTrim.show();
    }
    
    public void hide() {
        
        body.hide();
        sleeves.hide();
        neck.hide();
        bodyTrim.hide();
        neckTrim.hide();
        sleeveTrim.hide();
    }
    
    public DrawingCanvas getCanvas() {
        
        return body.getCanvas();
    }
    
    public void addToCanvas(DrawingCanvas c) {
        
        body.addToCanvas(c);
        sleeves.addToCanvas(c);
        neck.addToCanvas(c);
        bodyTrim.addToCanvas(c);
        neckTrim.addToCanvas(c);
        sleeveTrim.addToCanvas(c);
    }
}
